@@ -59,7 +59,6 @@ gint read_memswap(gulong *mem, gulong *swap, gulong *MT, gulong *MU, gulong *ST,
 {
     int fd;
     size_t n;
-    int o_MTotal, o_MFree, o_MBuffers, o_MCached, o_STotal, o_SFree;
     char *b_MTotal, *b_MFree, *b_MBuffers, *b_MCached, *b_STotal, *b_SFree;
 
     if ((fd = open("/proc/meminfo", O_RDONLY)) < 0)
@@ -78,28 +77,28 @@ gint read_memswap(gulong *mem, gulong *swap, gulong *MT, gulong *MU, gulong *ST,
     MemInfoBuf[n] = '\0';
 
     b_MTotal = strstr(MemInfoBuf, "MemTotal");
-    if (b_MTotal)
-        o_MTotal = sscanf(b_MTotal + strlen("MemTotal"), ": %lu", &MTotal);
+    if (!b_MTotal || !sscanf(b_MTotal + strlen("MemTotal"), ": %lu", &MTotal))
+        return -1;
 
     b_MFree = strstr(MemInfoBuf, "MemFree");
-    if (b_MFree)
-        o_MFree = sscanf(b_MFree + strlen("MemFree"), ": %lu", &MFree);
+    if (!b_MFree || !sscanf(b_MFree + strlen("MemFree"), ": %lu", &MFree))
+        return -1;
 
     b_MBuffers = strstr(MemInfoBuf, "Buffers");
-    if (b_MBuffers)
-        o_MBuffers = sscanf(b_MBuffers + strlen("Buffers"), ": %lu", &MBuffers);
+    if (!b_MBuffers || !sscanf(b_MBuffers + strlen("Buffers"), ": %lu", &MBuffers))
+        return -1;
 
     b_MCached = strstr(MemInfoBuf, "Cached");
-    if (b_MCached)
-        o_MCached = sscanf(b_MCached + strlen("Cached"), ": %lu", &MCached);
+    if (!b_MCached || !sscanf(b_MCached + strlen("Cached"), ": %lu", &MCached))
+        return -1;
 
     b_STotal = strstr(MemInfoBuf, "SwapTotal");
-    if (b_STotal)
-        o_STotal = sscanf(b_STotal + strlen("SwapTotal"), ": %lu", &STotal);
+    if (!b_STotal || !sscanf(b_STotal + strlen("SwapTotal"), ": %lu", &STotal))
+        return -1;
 
     b_SFree = strstr(MemInfoBuf, "SwapFree");
-    if (b_SFree)
-        o_SFree = sscanf(b_SFree + strlen("SwapFree"), ": %lu", &SFree);
+    if (!b_SFree || !sscanf(b_SFree + strlen("SwapFree"), ": %lu", &SFree))
+        return -1;
 
     MFree += MCached + MBuffers;
     MUsed = MTotal - MFree;
@@ -228,8 +227,8 @@ gint read_memswap(gulong *mem, gulong *swap, gulong *MT, gulong *MU, gulong *ST,
         return -1;
     }
 
-    *MT = (total_pages*pagesize) >> 10;
-    *MU = ((total_pages-free_pages-inactive_pages) * pagesize) >> 10;
+    *MT = CONVERT(total_pages);
+    *MU = CONVERT(total_pages-free_pages-inactive_pages);
     *mem = *MU * 100 / *MT;
 
     if((*swap = swapmode(&swap_avail, &swap_free)) >= 0) {
@@ -467,7 +466,68 @@ gint read_memswap(gulong *mem, gulong *swap, gulong *MT, gulong *MU, gulong *ST,
     return 0;
 }
 
-#else
-#error "Your platform is not yet support"
-#endif
+#elif defined (__sun__)
 
+#include <sys/stat.h>
+#include <sys/swap.h>
+#include <kstat.h>
+kstat_ctl_t *kc;
+
+static size_t MTotal = 0;
+static size_t MFree = 0;
+static size_t MUsed = 0;
+static size_t STotal = 0;
+static size_t SFree = 0;
+static size_t SUsed = 0;
+
+void mem_init_stats()
+{
+	kc = kstat_open();
+}
+
+gint read_memswap(gulong *mem, gulong *swap, gulong *MT, gulong *MU, gulong *ST, gulong *SU)
+{
+    long pagesize;
+    struct anoninfo swapinfo;
+    kstat_t *ksp;
+    kstat_named_t *knp;
+
+    pagesize = (long)(sysconf(_SC_PAGESIZE));
+
+    /* FIXME use real numbers, not fake data */
+    if (!kc)
+    {
+        mem_init_stats();
+    }
+
+    if (ksp = kstat_lookup(kc, "unix", 0, "system_pages"))
+    {
+        kstat_read(kc, ksp, NULL);
+	knp = kstat_data_lookup(ksp, "physmem");
+	MTotal = (pagesize * knp->value.ui64) >> 10;
+	knp = kstat_data_lookup(ksp, "freemem");
+	MUsed = MTotal - ((pagesize * knp->value.ui64) >> 10);
+    }
+    if (swapctl(SC_AINFO, &swapinfo) == 0) {
+        STotal = (swapinfo.ani_max * pagesize) >> 10;
+	SUsed = ((swapinfo.ani_max - swapinfo.ani_free) * pagesize) >> 10;
+	*swap = SUsed * 100 / STotal;
+    } else {
+        STotal = 0;
+	SUsed = 0;
+	*swap = 0;
+    }
+
+    *mem = MUsed * 100 / MTotal;
+
+    *MT = MTotal;
+    *MU = MUsed;
+    *ST = STotal;
+    *SU = SUsed;
+
+    return 0;
+}
+
+#else
+#error "Your platform is not yet supported"
+#endif
